
; Main CPU ATmega48
; DATE						
; steffen.hoerhold@web.de		
;								
; TAKT=1.0 - 8.0MHZ					
;								
.include "m48def.inc"
.include "defines.inc"				




.cseg			;Beginn eines Code-Segmentes
.org 0			;Startadresse=0				
; ***** INTERRUPT VECTORS ************************************************
rjmp 	MCU_RESTART     		; Reset Handler
reti 		;rjmp EXT_INT0		; External Interrupt Request 0
reti 		;rjmp EXT_INT1		; External Interrupt Request 1
reti 		;rjmp PC_INT0		; Pin Change Interrupt Request 0
rjmp 	PC_INT1		; Pin Change Interrupt Request 0
reti 		;rjmp PC_INT2		; Pin Change Interrupt Request 1
reti 		;rjmp WDT_TO		; Watchdog Time-out Interrupt
reti 		;rjmp TIMER2_COMPA	; Timer/Counter2 Compare Match A
reti 		;rjmp TIMER2_COMPB	; Timer/Counter2 Compare Match B
reti 		;rjmp TIMER2_OVF	; Timer/Counter2 Overflow
reti 		;rjmp TIMER1_ICP	; Timer/Counter1 Capture Event
rjmp 	TIMER1_COMPA	; Timer/Counter1 Compare Match A
reti 		;rjmp TIMER1_COMPB	; Timer/Counter1 Compare Match B
reti 		;rjmp TIMER1_OVF	; Timer/Counter1 Overflow
rjmp 	TIMER0_COMPA	; TimerCounter0 Compare Match A
reti 		;rjmp TIMER0_COMPB	; TimerCounter0 Compare Match B
reti 		;rjmp TIMER0_OVF	; Timer/Couner0 Overflow
reti 		;rjmp SPI_STC		; SPI Serial Transfer Complete
rjmp 	USART_RXC  				; USART RX Complete Handler
reti 		;rjmp USART_UDRE	; USART, Data Register Empty
reti 		;rjmp USART_TXC		; USART Tx Complete
reti 		;rjmp ADC_CC		; ADC Conversion Complete
reti 		;rjmp EE_RDY		; EEPROM Ready
reti 		;rjmp ANA_COMP		; Analog Comparator
reti 		;rjmp TWSI			; Two-wire Serial Interface
reti 		;rjmp SPM_RDY		; Store Program Memory Read
.org		INT_VECTORS_SIZE



;*************** INCLUDES ***************
.include "serial.inc"
.include "protocol.inc"
.include "settings.inc"
.include "nuts_bolts.inc"
.include "sw_spi.inc"
.include "sw_smarten.inc"
.include "watchdog.inc"
;****************************************

MCU_RESTART:                        
	
;STACK initialisieren
	clr		NULL
	ldi 	r16,low(RAMEND)			; Stackpointer initialisieren
	out 	SPL,r16					; muss in jedes Programm 
	ldi 	r16,high(RAMEND)		; als erstes mit eingebunden werden
	out 	SPH,r16

	rcall	watchdog_off

	sbi		EN_PORT,EN_PIN
	sbi		EN_PORT-1,EN_PIN
	sbi		LD2_PORT,LD2_PIN
	sbi		LD2_PORT-1,LD2_PIN


	ldi		r16,TICK_VAL
	ldi		r17,byte1(CURRENT_UP_VAL)
	ldi		r18,byte2(CURRENT_UP_VAL)

	load_p	X,(SRAM_START)
	ldi		r16,byte1(SRAM_SIZE)
clr_ram_loop:
	st		X+,NULL
	dec		r16
	brne	clr_ram_loop

	out		GPIOR0,NULL
	sts		GPIOR1,NULL
	sts		tst_mask,NULL

	rcall	init_uart
;	rcall	init_sw_spi
	rcall	init_hw_spi
	rcall	init_protocol
	
	sei

	rcall	init_tmc
	cbi		EN_PORT,EN_PIN
;	cbi		LD2_PORT,LD2_PIN


	rcall	init_smarten


main:
		sbic	GPIOR0,TICK_30us
		rcall	get_status
		rcall	protocol_process
		rjmp	main


get_status:
		cbi		GPIOR0,TICK_30us
		load_p	X,(status_position)
		rcall	spi_hw_send
		lds		r16,(status_rd +2)
		lds		r17,(tst_mask)
		or		r16,r17
		sbrc	r16,tmcOTPW
		sbi		GPIOR0,WARNING
		sbrs	r16,tmcOTPW
		cbi		GPIOR0,WARNING

		clr		r18
		cbi		GPIOR0,ERROR

		sbrc	r16,tmcS2SA
		inc		r18

		lds		r16,(status_rd +1)
		sbrc	r17,(tmcS2SA)
		rjmp	led_test_mode
		
		sbrc	r16,tmcS2GB
		inc		r18
		sbrc	r16,tmcOLA
		inc		r18
		sbrc	r16,tmcOLB
		inc		r18

		tst		r18
		breq	tmcSTST_check
		sbi		GPIOR0,ERROR

tmcSTST_check:
		sbrs	r16,tmcSTST
		rjmp	set_tmc_drive_current
		rjmp	set_tmc_hold_current
		rjmp	get_status_end

led_test_mode:
		sbi		GPIOR0,ERROR
		rjmp	tmcSTST_check

get_status_end:
		ret
		

set_tmc_drive_current:
		sbic	GPIOR0,DRV_CURRENT
		rjmp	inc_tmc_current_cnt					// inc actual current to drive current
		sbi		GPIOR0,CURRENT_STEP
		sbi		GPIOR0,DRV_CURRENT
		ldi		r16,step_CUR_UP
		sts		(seup_cnt),r16
inc_tmc_current_cnt:
		sbis	GPIOR0,CURRENT_STEP
		rjmp	set_tmc_drive_current_end			// reached drive current
inc_tmc_current:
		lds		r16,(cur_cnt)
		lds		r17,(curdrive)
		inc		r16
		cp		r16,r17
		brlo	set_new_drive_current		
reach_drive_current:
		cbi		GPIOR0,CURRENT_STEP
set_new_drive_current:
		sts		(tmc_current+2),r16
		sts		(cur_cnt),r16				
		load_p	X,(tmc_current)
		cbi		SPI_PORT,SPI_CS
		rcall	spi_hw_send
		sbi		SPI_PORT,SPI_CS
set_tmc_drive_current_end:
		ldi		r16,byte1(CURRENT_UP_VAL)
		ldi		r17,byte2(CURRENT_UP_VAL)
		sts		(cur_up_cnt+0),r16
		sts		(cur_up_cnt+1),r17
		rjmp	get_status_end
		ret


set_tmc_hold_current:	
		sbis	GPIOR0,DRV_CURRENT
		rjmp	dec_tmc_current_cnt			// dec actual current to hold current
		lds		r24,(cur_up_cnt+0)
		lds		r25,(cur_up_cnt+1)
		sbiw	r24:r25,1
		brne	store_cur_up_cnt
		sbi		GPIOR0,CURRENT_STEP
		cbi		GPIOR0,DRV_CURRENT
		ldi		r24,byte1(CURRENT_UP_VAL)
		ldi		r25,byte2(CURRENT_UP_VAL)
store_cur_up_cnt:
		sts		(cur_up_cnt+0),r24
		sts		(cur_up_cnt+1),r25				
dec_tmc_current_cnt:
		sbis	GPIOR0,CURRENT_STEP
		rjmp	set_tmc_hold_current_end	// reached hold current
		lds		r18,seup_cnt
		dec		r18
		breq	dec_tmc_current
		rjmp	store_seup_cnt
dec_tmc_current:
		ldi		r18,step_CUR_UP
		lds		r16,(cur_cnt)
		lds		r17,(curhold)
		dec		r16
		cp		r17,r16
		brlo	set_new_hold_current		
reach_hold_current:
		cbi		GPIOR0,CURRENT_STEP
set_new_hold_current:
		sts		(tmc_current+2),r16
		sts		(cur_cnt),r16				
		load_p	X,(tmc_current)
		cbi		SPI_PORT,SPI_CS
		rcall	spi_hw_send
		sbi		SPI_PORT,SPI_CS
store_seup_cnt:
		sts		seup_cnt,r18
set_tmc_hold_current_end:
		rjmp	get_status_end
		ret

