;--- Protokoll ---------------------------------------------------------
.equ	STATUS_OK 						= 0
.equ	STATUS_BAD_NUMBER_FORMAT		= 1
.equ	STATUS_EXPECTED_COMMAND_LETTER	= 2
.equ	STATUS_UNSUPPORTED_STATEMENT	= 3
.equ	STATUS_FLOATING_POINT_ERROR		= 4
.equ	STATUS_WITHOUT_STATEMENT		= 5


;------------------------------------------------------------------------------------------------------------------------
init_protocol:
		sts		char_counter,NULL		// Reset line input
		ldi		r16,byte1(CURRENT_UP_VAL)
		ldi		r17,byte2(CURRENT_UP_VAL)
		sts		(cur_up_cnt+0),r16
		sts		(cur_up_cnt+1),r17
		ldi		r16,step_CUR_UP
		mov		SEUP,r16
		sts		(seup_cnt),SEUP			// Current decrement steps
  // Check version-byte of eeprom
read_settings:
  		sbic	EECR,EEWE
		rjmp	read_settings
		out		EEARL,NULL
		sbi		EECR,EERE
		in		r16,EEDR
		cpi		r16, 0xFF//'1'				// EEPROM emty ???
		breq	first_settings
;		cpi		r16, '1'				// Version 1 ???
;		breq	first_settings
		rjmp	init_protocol_end
first_settings:
		clr		r17
		ldi		r18,9
		load_p	Z,(def_settings<<1)
		rjmp	def_eeprom_write
def_eeprom_write:
		sbic	EECR,EEPE
		rjmp	def_eeprom_write
		lpm		r16,Z+
		out		EEARL,r17
		inc		r17
		out		EEDR,r16
		sbi		EECR,EEMPE
		sbi		EECR,EEPE
		tst		r16
		brne	def_eeprom_write
		dec		r18
		brne	def_eeprom_write
		load_p	Z,(txt_epp_fail<<1)
		rcall	uart_fsend
		rcall	uart_send_enter
init_protocol_end:
		ldi		r16,1
//		ldi		r17,6
		rcall	read_eeprom_setting
		rcall	write_current_drive_reg
		lds		r16,(value+2)
		sts		(cur_cnt),r16
		ldi		r16,2
//		ldi		r17,6
		rcall	read_eeprom_setting
		rcall	write_status_wr_reg
		ldi		r16,5
//		ldi		r17,6
		rcall	read_eeprom_setting
		rcall	write_current_hold_reg
		rcall	dump_tmc_settings
		ldi		r16,(STATUS_OK)
		rcall	status_message
		ret

;------------------------------------------------------------------------------------------------------------------------
protocol_process:
		sbic	GPIOR0,TICK_30us
		rcall	get_status
		rcall   serial_read
        breq    protocol_process
		lds		r17,(char_counter)
; check \n
        cpi     r16,'\n'
		breq	rx_end_of_line_reached
; check \r
        cpi     r16,'\r'
		breq	rx_end_of_line_reached

		cpi		r16,2	// 2= STX
		breq	get_txt_string			// Throw away TEXT characters
        rjmp    rx_check_char
		
get_txt_string:
		lds		r18,GPIOR1
		sbr		r18,1<<(txt_string)
		sts		GPIOR1,r18
		rjmp	protocol_process_end

rx_end_of_line_reached:
		lds		r18,GPIOR1
		cbr		r18,1<<(txt_string)
		sts		GPIOR1,r18
		tst		r17						// (char_counter > 0) // Line is complete. Then execute!
		breq	skip_block				// Empty or comment line. Skip block.
		
		load_p	X,(line)
		add		XL,r17
		adc		XH,NULL			
		st		X,NULL					// Terminate string
		rcall	protocol_execute_line
		rcall	status_message
		rjmp	protocol_process_reset
skip_block:
		rcall	get_tmc_status
;		ldi		r16,(STATUS_WITHOUT_STATEMENT)			// Send status message for syncing purposes
;		rcall	status_message	
protocol_process_reset:
		sts		(char_counter),NULL		// Reset line buffer index
		lds		r18,GPIOR1
		cbr		r18,1<<(txt_string)
		sts		GPIOR1,r18	
		rjmp	protocol_process_end

rx_check_char:
		cpi		r16,0xFF
		breq	fw_update
; ? char_counter => LINE_BUFFER
		cpi		r17,LINE_BUFFER_SIZE-1
		brsh    protocol_process_end	// Throw away any characters beyond the end of the line buffer
; ? char <= 'STX'
		lds		r18,GPIOR1
		sbrc	r18,txt_string
		rjmp	store_line_char			
; ? char <= ' '
        cpi     r16,' '+1
        brlo    protocol_process_end	// Throw away whitepace and control characters
; ? (char_counter >= 'a' && char_counter <= 'z')
		cpi		r16,'z'+1
		brsh	protocol_process_end	//char_counter > 'z'
		cpi		r16,'a'
		brlo	store_line_char
		subi	r16, 'a'-'A'			// Upcase lowercase
		rjmp	store_line_char

		rjmp	protocol_process_end		
store_line_char:
		load_p	X,(line)
		add		XL,r17
		adc		XH,NULL
		inc		r17
		sts		(char_counter),r17
		st		X,r16
		rjmp	protocol_process_end
protocol_process_end:
		ret

fw_update:
		rcall	watchdog_on
fw_update_wait:
		rjmp	fw_update_wait
;------------------------------------------------------------------------------------------------------------------------
protocol_execute_line:
		load_p	X,(line)
		ld		r16,X+
		cpi		r16,'#'
		breq	protocol_execute_config
		cpi		r16,'W'
		breq	protocol_execute_warn
		cpi		r16,'E'
		breq	protocol_execute_err
		cpi		r16,'?'
		breq	stat_unsuported_statment
		cpi		r16,'I'
		breq	txt_execute_id
		cpi		r16,'R'
		breq	txt_execute_rs
		ldi		r16,(STATUS_UNSUPPORTED_STATEMENT)
		rjmp	protocol_execute_line_end
protocol_execute_config:
		ld		r16,X
		tst		r16
		brne	next_line_char
		rcall	dump_tmc_settings
		ldi		r16,(STATUS_OK)
		rjmp	protocol_execute_line_end
protocol_execute_warn:
		rcall	protocol_execute_warn_flag
		ldi		r16,(STATUS_OK)
		rjmp	protocol_execute_line_end
protocol_execute_err:
		rcall	protocol_execute_error_flag
		ldi		r16,(STATUS_OK)
		rjmp	protocol_execute_line_end
txt_execute_id:
		rjmp	jmp_txt_execute_id	
txt_execute_rs:
		rjmp	jmp_txt_execute_rs



next_line_char:
//read parameter
		rcall	read_parameter
		sbis	GPIOR0,0					
		rjmp	stat_bad_number_format		// return(false)
//read "="
		ld		r16,X+
		cpi		r16,'='
		brne	stat_unsuported_statment	// return(false)
//read value
		rcall	read_value
		sbis	GPIOR0,0
		rjmp	stat_bad_number_format		// return(false)
		rcall	store_tmc_settings
		rcall	write_tmc_setting
send_txt_store:
		load_p	Z,(txt_store<<1)
		rcall	uart_fsend
		rcall	uart_send_enter
		ldi		r16,(STATUS_OK)
		rjmp	protocol_execute_line_end

stat_unsuported_statment:
		ldi		r16,(STATUS_UNSUPPORTED_STATEMENT)
		rjmp	protocol_execute_line_end
stat_bad_number_format:
		ldi		r16,(STATUS_BAD_NUMBER_FORMAT)
		rjmp	protocol_execute_line_end



jmp_txt_execute_id:
//read I"D"
		ld		r16,X+
		cpi		r16,'D'
		brne	stat_unsuported_statment	// return(false)
//read ID"="
		ld		r16,X+
		cpi		r16,'='
		brne	stat_unsuported_statment	// return(false)

		load_p	Y,(value_ascii)
txt_execute_id_copy:
		ld		r16,X+
		st		Y+,r16
		tst		r16
		breq	txt_id_eep_store
		rjmp	txt_execute_id_copy
txt_id_eep_store:
		ldi		r16,7
//		ldi		r17,50
		rcall	write_eeprom_setting
		rjmp	send_txt_store

jmp_txt_execute_rs:
//read R"S"
		ld		r16,X+
		cpi		r16,'S'
		brne	stat_unsuported_statment	// return(false)
//read RS"="
		ld		r16,X+
		cpi		r16,'='
		brne	stat_unsuported_statment	// return(false)
		load_p	Y,(value_ascii)
		ldi		r17,5
txt_execute_rs_copy:
		ld		r16,X+
		tst		r16
		brne	txt_execute_rs_store
		ldi		r16,'0'
txt_rs_zero_loop:
		st		Y+,r16
		dec		r17
		brne	txt_rs_zero_loop
		rjmp	txt_rs_eep_store
txt_execute_rs_store:
		st		Y+,r16
		dec		r17
		brne	txt_execute_rs_copy
txt_rs_eep_store:
		st		Y+,NULL
		ldi		r16,6
		rcall	write_eeprom_setting
		rjmp	send_txt_store

protocol_execute_line_end:
		ret

;------------------------------------------------------------------------------------------------------------------------
protocol_execute_warn_flag:
		lds		r16,(tst_mask)
		sbrc	r16,tmcOTPW
		rjmp	protocol_warning_flag_off
		sbr		r16,1<<(tmcOTPW)
		rjmp	protocol_warning_flag_end
protocol_warning_flag_off:
		cbr		r16,1<<(tmcOTPW)
protocol_warning_flag_end:
		sts		(tst_mask),r16
		ret

;------------------------------------------------------------------------------------------------------------------------
protocol_execute_error_flag:
		lds		r16,(tst_mask)
		sbrc	r16,tmcS2SA
		rjmp	protocol_error_flag_off
		sbr		r16,1<<(tmcS2SA)
		rjmp	protocol_error_flag_end
protocol_error_flag_off:
		cbr		r16,1<<(tmcS2SA)
protocol_error_flag_end:
		sts		(tst_mask),r16
		ret

;------------------------------------------------------------------------------------------------------------------------
status_message:
		tst		r16
		breq	status_message_ok
		load_p	Z,(sc_error<<1)
		rcall	uart_fsend
		cpi		r16,STATUS_BAD_NUMBER_FORMAT
		breq	sm_bad_num_form
		cpi		r16,STATUS_EXPECTED_COMMAND_LETTER
		breq	sm_exp_com_form
		cpi		r16,STATUS_UNSUPPORTED_STATEMENT
		breq	sm_unsupport_state
		cpi		r16,STATUS_FLOATING_POINT_ERROR
		breq	sm_fpoint_err
		cpi		r16,STATUS_WITHOUT_STATEMENT
		breq	sm_without_statement
		rjmp	status_message_end

sm_bad_num_form:
		load_p	Z,(sc_bad_num_form<<1)
		rjmp	send_message
sm_exp_com_form:
		load_p	Z,(sc_exp_com_form<<1)
		rjmp	send_message
sm_unsupport_state:
		load_p	Z,(sc_unsupport_state<<1)
		rjmp	send_message
sm_fpoint_err:
		load_p	Z,(sc_fpoint_err<<1)
		rjmp	send_message
sm_without_statement:
		load_p	Z,(sc_without_state<<1)
		rjmp	send_message
status_message_ok:
		load_p	Z,(sc_ok<<1)
send_message:
		rcall	uart_fsend
status_message_end:
		rcall	uart_send_enter		
		ret

;------------------------------------------------------------------------------------------------------------------------		
sc_ok:				.db	"ok",0,0
sc_error:			.db	"error: ",0
sc_bad_num_form:	.db	"Bad number format",0
sc_exp_com_form:	.db	"Expected command letter",0
sc_unsupport_state:	.db	"Unsupported statement",0
sc_fpoint_err:		.db	"Floating point error",0,0
sc_without_state:	.db	"Without statement",0
