;--- sw_spi.inc --------------------------------------------------------------

init_sw_spi:
		in		r16,SPI_PORT
		cbr		r16,(1<<SPI_SDO)|(1<<SPI_SCK)|(1<<SPI_SDI)
		sbr		r16,(1<<SPI_CS)
        out     SPI_PORT,r16
		in		r16,SPI_PORT-1
		sbr		r16, (1<<SPI_SDO)|(1<<SPI_SCK)|(1<<SPI_CS)
		cbr		r16, (1<<SPI_SDI)	
        out     SPI_PORT-1,r16
		ret


init_hw_spi:
// configure the #SS line as an output for master
	sbi		SPI_PORT-1,SPI_CS
// Pull high the chip select line of the SPI serial memory		
	sbi		SPI_PORT,SPI_CS
// Port B use SPI master mode alternate function
// define the port direction : PB5/SCK, PB3/MOSI, PB2/!SS as outputs, otherwise PBx as input
	sbi		(SPI_PORT -1),SPI_SCK
	sbi		(SPI_PORT -1),SPI_SDO
// Disable the SPI to be able to configure the #SS line as an input even if the SPI is still configured as a slave
;	in		r16,SPCR
;	cbr		r16,(1<<SPE)
;	out		SPCR,r16

	in		r16,SPSR
.if SPI_DATA_RATE == 0
// data rate is Fosc/2
	sbr		r16, (1<<SPI2X)
.elif SPI_DATA_RATE == 1
// data rate is Fosc/4
	cbr		r16, (1<<SPI2X)
.elif SPI_DATA_RATE == 2
// data rate is Fosc/8
	sbr		r16, (1<<SPI2X)
.elif SPI_DATA_RATE == 3
// data rate is Fosc/16
	cbr		r16, (1<<SPI2X)
.elif SPI_DATA_RATE == 4
// data rate is Fosc/32
	sbr		r16, (1<<SPI2X)
.elif SPI_DATA_RATE == 5
// data rate is Fosc/64
	sbr		r16, (1<<SPI2X)
.else
// data rate is Fosc/128
	cbr		r16, (1<<SPI2X)
.endif
	out		SPSR,r16

// select the master mode prior to enable the SPI
	in		r16,SPCR
	sbr		r16, (1<<MSTR) + (1<<SPE)
//select the MSB first transmitted bit
	cbr		r16, (1<<DORD)
// SPI mode 0
	cbr		r16, (1<<CPOL) + (1<<CPHA)
.if SPI_DATA_RATE == 0
// data rate is Fosc/2
	cbr		r16, (1<<SPR1) + (1<<SPR0)
.elif SPI_DATA_RATE == 1
// data rate is Fosc/4
	cbr		r16, (1<<SPR1) + (1<<SPR0)
.elif SPI_DATA_RATE == 2
// data rate is Fosc/8
	cbr		r16, (1<<SPR1)
	sbr		r16, (1<<SPR0)
.elif SPI_DATA_RATE == 3
// data rate is Fosc/16
	cbr		r16, (1<<SPR1)
	sbr		r16, (1<<SPR0)
.elif SPI_DATA_RATE == 4
// data rate is Fosc/32
	sbr		r16, (1<<SPR1)
	cbr		r16, (1<<SPR0)
.elif SPI_DATA_RATE == 5
// data rate is Fosc/64
	sbr		r16, (1<<SPR1) + (1<<SPR0)
.else
// data rate is Fosc/128
	sbr		r16, (1<<SPR1) + (1<<SPR0)
.endif
	out		SPCR,r16	

	cli
// Enable the SPI interrupt line,
;	in		r16,SPCR
;	sbr		r16, (1<<SPIE)
;	out		SPCR,r16		        
// Clear the SPIF flag
	in		r16, SPSR
	in		r16, SPDR
	sei
	ret


;***************************** SPI Send 24bit ******************************
; Sub: Sendet 24Bit Kommando aus SRAM
; Parameter:	X (pointer to SPI-Data in SRAM to write)
; Return:		readed SPI-Data in SRAM
; Scratch-Reg:	r16
;***************************************************************************
;------------------------------------------------------------------------------------------------------------------------
spi_send_old:
		push	r16
		load_p	Y,(status_rd)
		cbi		SPI_PORT,SPI_CS				; CS aktiv
		ld		r16,X+
		rcall	spi_send_byte
		st		Y+,r16
		ld		r16,X+
		rcall	spi_send_byte
		st		Y+,r16
		ld		r16,X+
		rcall	spi_send_byte
		st		Y+,r16
		sbi		SPI_PORT,SPI_CS				; CS wieder H
		pop		r16
		ret



spi_hw_send:
		push	r16
		push	r17
		push	YL
		push	YH
		ldi		r17,3
		load_p	Y,(status_rd)
		cbi		SPI_PORT,SPI_CS				; CS aktiv
spi_hw_send_byte_loop:
;		lds		r16,mSPSR
;		sbrs	r16,SPIF
;		rjmp	spi_hw_send_byte_loop
		ld		r16,X+
		out		SPDR,r16
spi_hw_read_byte_loop:
		in		r16,SPSR
		sbrs	r16,SPIF
		rjmp	spi_hw_read_byte_loop
		in		r16,SPDR
		st		Y+,r16
		dec		r17
		brne	spi_hw_send_byte_loop
		sbi		SPI_PORT,SPI_CS				; CS wieder H
		pop		YH
		pop		YL
		pop		r17
		pop		r16
		ret

;--- sw_spi.inc --------------------------------------------------------------

;******************************* SPI Send byte *****************************
; Sub: send one byte by r16
; Parameter:	r16 write data
; Return:		r16 readed data
; Scratch-Reg:	-
;***************************************************************************
;------------------------------------------------------------------------------------------------------------------------
spi_send_byte:
spi_send:
		push	r17
		push	r18
		ldi		r18,3
		load_p	Y,(status_rd)
		cbi		SPI_PORT,SPI_CS				; CS aktiv
spi_send_bytes:
		ld		r16,X+
		sec
		rol		r16
		rjmp	spi_send_bit1
spi_send_byte_loop:
		lsl		r16
spi_send_bit1:
		breq	spi_send_byte_end
		brcc	spi_send_lo_bit
		sbi		SPI_PORT,SPI_SDO			; Daten auf H
		rjmp	spi_send_clk_hi
spi_send_lo_bit:
		cbi		SPI_PORT,SPI_SDO			; nein, also auf L
spi_send_clk_hi:
		sbi		SPI_PORT,SPI_SCK			; Clock auf H
		clc
		sbic	SPI_PORT-2,SPI_SDI
		sec										; Carry setzen
		rol		r17								; in Rckgabewert reinschieben
		cbi		SPI_PORT,SPI_SCK			; Clock wieder L
		rjmp	spi_send_byte_loop
spi_send_byte_end:
		st		Y+,r17
		dec		r18
		brne	spi_send_bytes
		sbi		SPI_PORT,SPI_CS				; CS aktiv
		pop		r18
		pop		r17
		ret
